package com.fjhx.item.controller.product;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.item.entity.product.dto.ProductInfoDto;
import com.fjhx.item.entity.product.dto.ProductInfoSelectDto;
import com.fjhx.item.entity.product.vo.ProductInfoVo;
import com.fjhx.item.service.product.ProductInfoService;
import com.ruoyi.common.core.domain.BaseSelectDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 产品库 前端控制器
 * </p>
 *
 * @author zlj
 * @since 2023-03-17
 */
@RestController
@RequestMapping("/productInfo")
public class ProductInfoController {

    @Autowired
    private ProductInfoService productInfoService;

    /**
     * 产品库分页
     */
    @PostMapping("/page")
    public Page<ProductInfoVo> page(@RequestBody ProductInfoSelectDto dto) {
        return productInfoService.getPage(dto);
    }


    /**
     * 产品库统计(根据产品库类型进行分类统计)
     */
    @PostMapping("/productInfoStatistics")
    public Map<String, Object> productInfoStatistics(@RequestBody ProductInfoSelectDto dto) {
        return productInfoService.productInfoStatistics(dto);
    }

    /**
     * 产品库明细
     */
    @PostMapping("/detail")
    public ProductInfoVo detail(@RequestBody BaseSelectDto dto) {
        return productInfoService.detail(dto.getId());
    }

    /**
     * 产品库新增
     */
    @PostMapping("/add")
    public void add(@RequestBody ProductInfoDto productInfoDto) {
        productInfoService.add(productInfoDto);
    }

    /**
     * 产品库编辑
     */
    @PostMapping("/edit")
    public void edit(@RequestBody ProductInfoDto productInfoDto) {
        productInfoService.edit(productInfoDto);
    }

    /**
     * 产品库删除
     */
    @PostMapping("/delete")
    public void delete(@RequestBody BaseSelectDto dto) {
        productInfoService.delete(dto.getId());
    }

    /**
     * 产品统计（数据看板-产品分析页面）
     *
     * @param productInfoDto
     */
    @PostMapping("/productStatistics")
    public Map<String, Object> productStatistics(@RequestBody ProductInfoDto productInfoDto) {
        return productInfoService.productStatistics(productInfoDto);
    }

    /**
     * 产品分布（数据看板-产品分析页面）
     *
     * @param productInfoDto
     */
    @PostMapping("/productDistribution")
    public List<Map<String, Object>> productDistribution(@RequestBody ProductInfoDto productInfoDto) {
        return productInfoService.productDistribution(productInfoDto);
    }

    /**
     * 产品类型排行（数据看板-产品分析页面）
     *
     * @param productInfoDto
     */
    @PostMapping("/productTypeRanking")
    public List<Map<String, Object>> productTypeRanking(@RequestBody ProductInfoSelectDto productInfoDto) {
        return productInfoService.productTypeRanking(productInfoDto);
    }

    /**
     * 产品排行（数据看板-产品分析页面）
     *
     * @param productInfoDto
     */
    @PostMapping("/productRanking")
    public Page<ProductInfoVo> productRanking(@RequestBody ProductInfoSelectDto productInfoDto) {
        return productInfoService.productRanking(productInfoDto);
    }

    /**
     * 产品类型统计
     */
    @GetMapping("/statisticsProduct")
    public Map<String, Object> statisticsProduct() {
        return productInfoService.statisticsProduct();
    }

    /**
     * 导出Excel
     */
    @PostMapping("/exportExcel")
    public void excelExport(HttpServletResponse httpServletResponse, @RequestBody ProductInfoSelectDto dto) {
        productInfoService.excelExport(httpServletResponse, dto);
    }

}
