package com.fjhx.wms.service.stock.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.entity.InOutBo;
import com.fjhx.common.utils.Assert;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.wms.entity.stock.dto.StockJournalDetailsDto;
import com.fjhx.wms.entity.stock.dto.StockWaitDetailsDto;
import com.fjhx.wms.entity.stock.dto.StockWaitDto;
import com.fjhx.wms.entity.stock.dto.StockWaitSelectDto;
import com.fjhx.wms.entity.stock.emums.JournalType;
import com.fjhx.wms.entity.stock.po.*;
import com.fjhx.wms.entity.stock.vo.StockWaitDetailsVo;
import com.fjhx.wms.entity.stock.vo.StockWaitVo;
import com.fjhx.wms.entity.warehouse.po.Warehouse;
import com.fjhx.wms.mapper.stock.StockWaitMapper;
import com.fjhx.wms.service.MyPurchaseService;
import com.fjhx.wms.service.stock.*;
import com.fjhx.wms.service.warehouse.WarehouseService;
import com.fjhx.wms.utils.CodeEnum;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 待出入库 服务实现类
 * </p>
 *
 * @author
 * @since 2023-03-22
 */
//@DS(SourceConstant.WMS)
@Service
public class StockWaitServiceImpl extends ServiceImpl<StockWaitMapper, StockWait> implements StockWaitService {
    @Autowired
    private ProductInfoService productInfoService;
    @Autowired
    private StockService stockService;
    @Autowired
    private StockJournalService stockJournalService;
    @Autowired
    private StockJournalDetailsService stockJournalDetailsService;
    @Autowired
    private StockWaitDetailsService stockWaitDetailsService;
    @Autowired
    private WarehouseService warehouseService;
    @Autowired
    private MyPurchaseService myPurchaseService;

    @Override
    public Page<StockWaitVo> getPage(StockWaitSelectDto dto) {
        IWrapper<StockWait> wrapper = getWrapper();
        //过滤入库/出库
        wrapper.eq(StockWait::getType, dto.getType());
        wrapper.eq("sw", StockWait::getBusinessType, dto.getBusinessType());
        wrapper.eq(StockWait::getBusinessCode, dto.getKeyword());
        wrapper.orderByDesc("sw", StockWait::getId);
        Page<StockWaitVo> page = this.baseMapper.getPage1(dto.getPage(), wrapper);

        List<StockWaitVo> records = page.getRecords();
        // 赋值产品属性
        productInfoService.attributeAssign(records, StockWaitVo::getProductId, (item, productInfo) -> {
            item.setProductCode(productInfo.getCode());
            item.setProductUnit(productInfo.getUnit());
            item.setProductType(productInfo.getType());
            item.setProductName(productInfo.getName());
            item.setProductSpec(productInfo.getSpec());
        });

        return page;
    }

//    @Override
//    public Page<StockWaitVo> getPageByWdly(StockWaitSelectDto dto) {
//        IWrapper<StockWait> wrapper = getWrapper();
//        wrapper.orderByDesc("sw", StockWait::getId);
//        //过滤入库/出库
//        wrapper.eq(StockWait::getType, dto.getType());
//        wrapper.eq(StockWait::getBusinessCode, dto.getKeyword());
//        wrapper.eq(StockWait::getStatus, dto.getStatus());
//        wrapper.eq("sw", StockWait::getBusinessType, dto.getBusinessType());
//
//        //过滤掉待入库待出库已完成数据
//        wrapper.ne("sw", StockWait::getStatus, 2);
//
//        Page<StockWaitVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
//
//        List<StockWaitVo> records = page.getRecords();
//        //赋值操作人
//        UserUtil.assignmentNickName(records, StockWaitVo::getCreateUser, StockWaitVo::setOperatorName);
//        //赋值仓库名称和id
//        List<Long> warehouseIds = new ArrayList<>();
//        for (StockWaitVo stockWaitVo : records) {
//            String victoriatouristJson = stockWaitVo.getVictoriatouristJson();
//            if (ObjectUtil.isEmpty(victoriatouristJson)) {
//                continue;
//            }
//            JSONObject json = JSONObject.parseObject(victoriatouristJson);
//            if (!json.containsKey("receiptWarehouseId")) {
//                continue;
//            }
//            warehouseIds.add(json.getLong("receiptWarehouseId"));
//        }
//        if (ObjectUtil.isEmpty(warehouseIds)) {
//            return page;
//        }
//        List<Warehouse> warehouses = warehouseService.listByIds(warehouseIds);
//        Map<Long, String> warehousesMap = warehouses.stream().collect(Collectors.toMap(Warehouse::getId, Warehouse::getName));
//        for (StockWaitVo stockWaitVo : records) {
//            String victoriatouristJson = stockWaitVo.getVictoriatouristJson();
//            if (ObjectUtil.isEmpty(victoriatouristJson)) {
//                continue;
//            }
//            JSONObject json = JSONObject.parseObject(victoriatouristJson);
//            json.put("receiptWarehouseName", warehousesMap.get(json.getLong("receiptWarehouseId")));
//            stockWaitVo.setVictoriatouristJson(JSONObject.toJSONString(json, JSONWriter.Feature.WriteLongAsString));
//        }
//
//        return page;
//    }

    @Override
    public StockWaitVo detail(Long id) {
        StockWait stockWait = this.getById(id);
        StockWaitVo result = BeanUtil.toBean(stockWait, StockWaitVo.class);

        StockWaitDetails stockWaitDetails = stockWaitDetailsService.getOne(q -> q.eq(StockWaitDetails::getStockWaitId, id));
        ProductInfo byId = productInfoService.getById(stockWaitDetails.getProductId());
        result.setProductName(byId.getName());
        result.setQuantity(stockWaitDetails.getQuantity());
        result.setStockWaitDetailsId(stockWaitDetails.getId());

        //获取明细列表
        List<StockWaitDetails> stockWaitDetailsList = stockWaitDetailsService.list(q -> q.eq(StockWaitDetails::getStockWaitId, id));
        List<StockWaitDetailsVo> stockWaitDetailsVos = BeanUtil.copyToList(stockWaitDetailsList, StockWaitDetailsVo.class);
        //赋值产品信息
        productInfoService.attributeAssign(stockWaitDetailsVos, StockWaitDetailsVo::getProductId, (item, productInfo) -> {
            item.setProductName(productInfo.getName());
        });
        result.setStockWaitDetailsList(stockWaitDetailsVos);

        return result;
    }

    @DSTransactional
    @Override
    public void add(StockWaitDetailsDto stockWaitDetailsDto) {
        //优化
        StockWaitDetails stockWaitDetails = stockWaitDetailsService.getById(stockWaitDetailsDto.getId());
        Assert.notEmpty(stockWaitDetails, "查询不到待出库明细信息");
        StockWaitDto stockWaitDto = BeanUtil.copyProperties(stockWaitDetailsDto, StockWaitDto.class);
        stockWaitDto.setId(stockWaitDetails.getStockWaitId());
        stockWaitDto.setStockWaitDetailsList(Collections.singletonList(BeanUtil.copyProperties(stockWaitDetailsDto, StockWaitDetails.class)));

        Long warehouseId = stockWaitDto.getWarehouseId();
        Warehouse warehouse = warehouseService.getById(warehouseId);
        if (ObjectUtil.isEmpty(warehouse)) {
            throw new ServiceException("查询不到仓库信息！");
        }
        Long companyId = warehouse.getCompanyId();

        Assert.notEmpty(stockWaitDto.getId(), "待出入库id不能为空");
        Assert.notEmpty(warehouseId, "仓库id不能为空");

        List<Stock> stockList = new ArrayList<>();

        List<StockWaitDetails> stockWaitDetailsDtoList = stockWaitDto.getStockWaitDetailsList();

        //更新已出入库数量以及待出入库状态
        stockWaitDetailsService.changeStockWaitStatus(stockWaitDto);
        StockWait stockWait = getById(stockWaitDto.getId());
        updateById(stockWait);

        //操作库存
        List<Long> swdIds = stockWaitDetailsDtoList.stream().map(StockWaitDetails::getId).collect(Collectors.toList());
        Map<Long, StockWaitDetails> oldStockWaitDetailsMap = stockWaitDetailsService.mapKEntity(StockWaitDetails::getId, q -> q.in(StockWaitDetails::getId, swdIds));

        //创建出入库记录
        Integer businessType = stockWait.getBusinessType();
        JournalType journalType = JournalType.getByDetailType(businessType);
        StockJournal stockJournal = new StockJournal();
        stockJournal.setOpType(stockWait.getType());//根据待入库类型赋值操作类型 1入库 2出库
        stockJournal.setType(journalType.getDetailType());
        stockJournal.setCode(stockWait.getType() == 1 ? CodeEnum.SIN_CODE.getCode() : CodeEnum.SOUT_CODE.getCode());
        stockJournal.setWarehouseId(warehouseId);
        stockJournal.setBusinessId(stockWaitDto.getId());
        stockJournal.setCompanyId(companyId);
        //保存出入库记录
        stockJournalService.save(stockJournal);
        List<StockJournalDetails> stockJournalDetailsList = new ArrayList<>();
        for (StockWaitDetails waitDetails : stockWaitDetailsDtoList) {
            StockWaitDetails temp = oldStockWaitDetailsMap.get(waitDetails.getId());
            Stock stock = new Stock();
            stock.setQuantity(waitDetails.getQuantity());
            stock.setProductId(temp.getProductId());
            stockList.add(stock);

            //创建出入库明细
            StockJournalDetails stockJournalDetails = new StockJournalDetailsDto();
            stockJournalDetails.setStockJournalId(stockJournal.getId());
            stockJournalDetails.setProductId(stock.getProductId());
            stockJournalDetails.setQuantity(stock.getQuantity());
            stockJournalDetails.setBusinessDetailsId(waitDetails.getId());
            stockJournalDetails.setCompanyId(companyId);
            stockJournalDetailsList.add(stockJournalDetails);

        }
        List<InOutBo> inOutBoList = BeanUtil.copyToList(stockList, InOutBo.class);

        //操作库存根据待出入库类型来1入库2出库
        stockService.changeStock(inOutBoList, warehouseId, journalType);
        //保存出入库明细
        stockJournalDetailsService.saveBatch(stockJournalDetailsList);

        //如果是采购退货 修改采购退货状态
        Assert.notEmpty(stockWait, "查询不到待出库信息");
        if (stockWait.getBusinessType() == 4) {
            Integer status = stockWait.getStatus();
            myPurchaseService.updateSalesReturnStatus(stockWaitDetails.getBusinessDetailsId(), status);
        }
    }

    @Override
    public void edit(StockWaitDto stockWaitDto) {
        this.updateById(stockWaitDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
